"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanOrphans = exports.cleanExpiredDlxCache = void 0;
const fs_1 = require("fs");
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("util"));
async function cleanExpiredDlxCache({ cacheDir, dlxCacheMaxAge, now, }) {
    if (dlxCacheMaxAge === Infinity)
        return;
    const dlxCacheDir = path_1.default.join(cacheDir, 'dlx');
    const dlxCacheNames = readOptDir(dlxCacheDir);
    if (!dlxCacheNames)
        return;
    await Promise.all(dlxCacheNames.map(async (dlxCacheName) => {
        const dlxCachePath = path_1.default.join(dlxCacheDir, dlxCacheName);
        const dlxCacheLink = path_1.default.join(dlxCachePath, 'pkg');
        let shouldClean;
        if (dlxCacheMaxAge <= 0) {
            shouldClean = true;
        }
        else {
            const dlxCacheLinkStats = await getStats(dlxCacheLink);
            shouldClean = dlxCacheLinkStats !== 'ENOENT' && isOutdated(dlxCacheLinkStats, dlxCacheMaxAge, now);
        }
        if (shouldClean) {
            // delete the symlink, the symlink's target, and orphans (if any)
            await promises_1.default.rm(dlxCachePath, { recursive: true });
        }
    }));
    await cleanOrphans(dlxCacheDir);
}
exports.cleanExpiredDlxCache = cleanExpiredDlxCache;
async function cleanOrphans(dlxCacheDir) {
    const dlxCacheNames = readOptDir(dlxCacheDir);
    if (!dlxCacheNames)
        return;
    await Promise.all(dlxCacheNames.map(async (dlxCacheName) => {
        const dlxCachePath = path_1.default.join(dlxCacheDir, dlxCacheName);
        const dlxCacheLink = path_1.default.join(dlxCachePath, 'pkg');
        const dlxCacheLinkStats = await getStats(dlxCacheLink);
        if (dlxCacheLinkStats === 'ENOENT') {
            return promises_1.default.rm(dlxCachePath, { recursive: true });
        }
        const dlxCacheLinkTarget = await getRealPath(dlxCacheLink);
        const children = await promises_1.default.readdir(dlxCachePath);
        await Promise.all(children.map(async (name) => {
            if (name === 'pkg')
                return;
            const fullPath = path_1.default.join(dlxCachePath, name);
            if (fullPath === dlxCacheLinkTarget)
                return;
            await promises_1.default.rm(fullPath, { recursive: true });
        }));
    }));
}
exports.cleanOrphans = cleanOrphans;
function isOutdated(stats, dlxCacheMaxAge, now) {
    return stats.mtime.getTime() + dlxCacheMaxAge * 60_000 < now.getTime();
}
async function getStats(path) {
    try {
        return await promises_1.default.lstat(path);
    }
    catch (err) {
        if (util_1.default.types.isNativeError(err) && 'code' in err && err.code === 'ENOENT') {
            return 'ENOENT';
        }
        throw err;
    }
}
function readOptDir(dirPath) {
    try {
        return (0, fs_1.readdirSync)(dirPath, 'utf-8');
    }
    catch (err) {
        if (util_1.default.types.isNativeError(err) && 'code' in err && err.code === 'ENOENT') {
            return null;
        }
        throw err;
    }
}
async function getRealPath(linkPath) {
    try {
        return await promises_1.default.realpath(linkPath);
    }
    catch (err) {
        if (util_1.default.types.isNativeError(err) && 'code' in err && err.code === 'ENOENT') {
            return null;
        }
        throw err;
    }
}
//# sourceMappingURL=cleanExpiredDlxCache.js.map